/** Rill v0.0.672 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, element, expect, externalRelReg, handleClick, handleSubmit, handleURLChange, handler, hashReg, initialized, parseURL, referrer, serialize;

serialize = require("@rill/form-json");

parseURL = require("@rill/parse-url");

expect = require("../util/expect");

referrer = document.referrer;

hashReg = /#.+$/;

externalRelReg = /(?:^|\s+)external(?:\s+|$)/;

initialized = false;

handler = function() {};

element = null;

browser = {

  /*
  	 * Listen to all url change events on a dom element and trigger the server callback.
  	 *
  	 * @param {DOMNode} element
  	 * @api private
   */
  listen: function(entity) {
    if (entity == null) {
      entity = global;
    }
    expect(element == null, "Can only run one server at a time in the browser.");
    element = entity;
    global.addEventListener("DOMContentLoaded", handleURLChange);
    global.addEventListener("popstate", handleURLChange);
    element.addEventListener("submit", handleSubmit);
    element.addEventListener("click", handleClick);
  },

  /*
  	 * Add a handler to listen to url changes.
  	 *
  	 * @param {Function} fn
  	 * @api private
   */
  createServer: function(fn) {
    handler = fn;
    return browser;
  },

  /*
  	 * Trigger the registered handler to navigate to a given url.
  	 *
  	 * @param {String} url
  	 * @param {Object} options
  	 * @api private
   */
  goTo: function(req, replaceState) {
    var res;
    if (typeof req === "string") {
      req = {
        url: req
      };
    }
    if (req.method == null) {
      req.method = "GET";
    }
    if (req.headers == null) {
      req.headers = {};
    }
    req.headers["cookie"] = document.cookie;
    req.headers["user-agent"] = navigator.userAgent;
    req.headers["referer"] = referrer;
    referrer = req.url;
    req.connection = {
      remoteAddress: "127.0.0.1"
    };
    req.socket = {
      encrypted: location.protocol === "https:"
    };
    expect(req.url, "A url is required.");
    return handler(req, res = {}).then(function() {
      var hash, ref, target;
      if (req.method !== "GET" || res.headers["location"]) {
        return;
      }
      hash = (ref = req.url.match(hashReg)) != null ? ref[0] : void 0;

      /*
      			 * When navigating a user will be brought to the top of the page.
      			 * If the urls contains a hash that is the id of an element (a target) then the target will be scrolled to.
      			 * This is similar to how browsers handle page transitions natively.
       */
      if (hash) {
        target = document.getElementById(hash.slice(1));
        if (target) {
          target.scrollIntoView({
            block: "start",
            behavior: "smooth"
          });
        }
      } else if (initialized) {
        global.scrollTo(0, 0);
      }
      initialized = true;
      history[(replaceState ? "replaceState" : "pushState")](null, "", req.url);
    });
  }
};


/*
 * Handle an event that changed the url (popstate or page load)
 *
 * @param {Object} event
 */

handleURLChange = function(event) {
  return browser.goTo(location.href, true);
};


/*
 * Handle intercepting link clicks to update the url.
 *
 * @param {Object} event
 */

handleClick = function(event) {
  var el, host, protocol, ref, url;
  if (event.defaultPrevented || event.metaKey || event.ctrlKey || event.shiftKey || event.button !== 0) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "A") {
    el = el.parentNode;
  }
  url = el != null ? el.href : void 0;
  if ((url == null) || el.attributes.download || (el.target && el.target !== "_self") || (el.rel && externalRelReg.test(el.rel)) || ((ref = parseURL(url), protocol = ref.protocol, host = ref.host, ref), location.protocol.indexOf(protocol) === -1 || location.host !== host)) {
    return;
  }
  browser.goTo(url);
  return event.preventDefault();
};


/*
 * Handle intercepting forms to update the url.
 *
 * @param {Object} event
 */

handleSubmit = function(event) {
  var body, el, files, host, method, protocol, ref, ref1, url;
  if (event.defaultPrevented) {
    return;
  }
  el = event.target;
  while ((el != null) && el.nodeName !== "FORM") {
    el = el.parentNode;
  }
  url = el != null ? el.action : void 0;
  if ((url == null) || (el.target && el.target !== "_self") || (el.rel && externalRelReg.test(el.rel)) || ((ref = parseURL(url), protocol = ref.protocol, host = ref.host, ref), location.protocol.indexOf(protocol) === -1 || location.host !== host)) {
    return;
  }
  ref1 = serialize(el), body = ref1.body, files = ref1.files;
  method = (el.getAttribute("method") || el.method).toUpperCase();
  if (method === "GET") {
    url = (url.split("?")[0]) + "?" + (JSON.stringify(body));
    body = {};
    files = [];
  }
  browser.goTo({
    url: url,
    method: method,
    body: body,
    files: files
  });
  if (!el.hasAttribute("data-noreset")) {
    el.reset();
  }
  return event.preventDefault();
};

module.exports = browser;
