/** Rill v0.0.672 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var browser, byteLength, htmlReg, isType, statuses;

statuses = require("statuses");

isType = require("@rill/is-type");

byteLength = require("./byteLength");

htmlReg = /^\s*</;


/*
 * A default middleware that handles sending the response for the request.
 *
 * @param {Object} req
 * @param {Object} res
 */

module.exports = function(req, res) {
  var body, headers, originalRes, status, statusText;
  if (res.headersSent) {
    return;
  }
  originalRes = res.res, headers = res.headers, status = res.status, statusText = res.statusText, body = res.body;
  req.cookies.save();
  if (headers["location"] != null) {
    if (!statuses.redirect[status]) {
      status = 302;
    }
  }
  if (Number(status) === 404 && (body != null)) {
    status = 200;
  }
  if (statusText == null) {
    statusText = statuses[status];
  }
  switch (false) {
    case !(statuses.empty[status] || (body == null)):
      delete headers["content-type"];
      body = void 0;
      break;
    case !isType.String(body):
      if (headers["content-type"] == null) {
        headers["content-type"] = "text/" + (htmlReg.test(body) ? "html" : "plain") + "; charset=UTF-8";
      }
      break;
    case !isType.Buffer(body):
      headers["content-length"] = body.length;
      break;
    case !isType.Stream(body):
      if (headers["content-type"] == null) {
        headers["content-type"] = "application/octet-stream";
      }
      break;
    default:
      try {
        body = JSON.stringify(body);
        if (headers["content-type"] == null) {
          headers["content-type"] = "application/json; charset=UTF-8";
        }
      } catch (_error) {}
  }
  {
    originalRes.writeHead(status, statusText, headers);
    switch (false) {
      case "HEAD" !== req.method:
        originalRes.end();
        break;
      case !isType.Stream(body):
        body.pipe(originalRes);
        break;
      default:
        if (headers["content-length"] == null) {
          headers["content-length"] = byteLength(body);
        }
        originalRes.end(body);
    }
  }
};
