serialize      = require("@rill/form-json")
parseURL       = require("@rill/parse-url")
expect         = require("../util/expect")
referrer       = document.referrer
hashReg        = /#.+$/
externalRelReg = /(?:^|\s+)external(?:\s+|$)/
initialized    = false

# Store registered createServer callback.
handler = ->
element = null

browser =
	###
	# Listen to all url change events on a dom element and trigger the server callback.
	#
	# @param {DOMNode} element
	# @api private
	###
	listen: (entity = global)->
		expect(not element?, "Can only run one server at a time in the browser.")
		element = entity
		global.addEventListener("DOMContentLoaded", handleURLChange)
		global.addEventListener("popstate", handleURLChange)
		element.addEventListener("submit", handleSubmit)
		element.addEventListener("click", handleClick)
		return

	###
	# Add a handler to listen to url changes.
	#
	# @param {Function} fn
	# @api private
	###
	createServer: (fn)->
		handler = fn
		return browser

	###
	# Trigger the registered handler to navigate to a given url.
	#
	# @param {String} url
	# @param {Object} options
	# @api private
	###
	goTo: (req, replaceState)->
		req = { url: req } if typeof req is "string"
		req.method               ?= "GET"
		req.headers              ?= {}
		req.headers["cookie"]     = document.cookie
		req.headers["user-agent"] = navigator.userAgent
		req.headers["referer"]    = referrer; referrer = req.url # Why am I subject to "referer" :(.
		req.connection            = remoteAddress: "127.0.0.1"
		req.socket                = encrypted: location.protocol is "https:"

		expect(req.url, "A url is required.")

		# Create a mock http request to bootstrap the browser.
		handler(req, res = {}).then(->
			# Update browser url on get requests.
			return if req.method isnt "GET" or res.headers["location"]
			hash = req.url.match(hashReg)?[0]

			###
			# When navigating a user will be brought to the top of the page.
			# If the urls contains a hash that is the id of an element (a target) then the target will be scrolled to.
			# This is similar to how browsers handle page transitions natively.
			###
			if hash
				target = document.getElementById(hash[1..])
				target.scrollIntoView(block: "start", behavior: "smooth") if target
			else if initialized
				global.scrollTo(0, 0)

			initialized = true
			history[(
				if replaceState then "replaceState"
				else "pushState"
			)](null, "", req.url)
			return
		)

###
# Handle an event that changed the url (popstate or page load)
#
# @param {Object} event
###
handleURLChange = (event)->
	browser.goTo(location.href, true)

###
# Handle intercepting link clicks to update the url.
#
# @param {Object} event
###
handleClick = (event)->
	# Ignore canceled events, modified clicks, and right clicks.
	return if event.defaultPrevented or
			  event.metaKey  or
			  event.ctrlKey  or
			  event.shiftKey or
			  event.button   isnt 0

	# Get the <a> element.
	el  = event.target
	el  = el.parentNode while el? and el.nodeName isnt "A"
	url = el?.href


	# Ensure that this element meets some criteria.
	return if (
		# Ignore clicks from linkless elements
		(not url?) or
		# Ignore the click if it's a download link. (We use this method of
		# detecting the presence of the attribute for old IE versions.)
		(el.attributes.download) or
		# Ignore the click if the element has a target.
		(el.target and el.target isnt "_self") or
		# Ignore 'rel="external"' links.
		(el.rel and externalRelReg.test(el.rel)) or
		# Ignore links that don't share a protocol and host with ours.
		(
			# Use a url parser to parse URLs instead of relying on the browser
			# to do it for us (because IE).
			{ protocol, host } = parseURL(url)
			location.protocol.indexOf(protocol) is -1 or location.host isnt host
		)
	)

	browser.goTo(url)
	event.preventDefault()

###
# Handle intercepting forms to update the url.
#
# @param {Object} event
###
handleSubmit = (event)->
	# Ignore canceled events.
	return if event.defaultPrevented

	# Get the <form> element.
	el  = event.target
	el  = el.parentNode while el? and el.nodeName isnt "FORM"
	url = el?.action

	# Ensure that this element meets some criteria.
	return if (
		# Ignore clicks from linkless elements
		(not url?) or
		# Ignore the click if the element has a target.
		(el.target and el.target isnt "_self") or
		# Ignore 'rel="external"' links.
		(el.rel and externalRelReg.test(el.rel)) or
		# Ignore links that don't share a protocol and host with ours.
		(
			# Use a url parser to parse URLs instead of relying on the browser
			# to do it for us (because IE).
			{ protocol, host } = parseURL(url)
			location.protocol.indexOf(protocol) is -1 or location.host isnt host
		)
	)

	{ body, files } = serialize(el)
	method          = (el.getAttribute("method") or el.method).toUpperCase()

	if method is "GET"
		# Ignore qs when using GET (consistent with browsers).
		url   = "#{url.split("?")[0]}?#{JSON.stringify(body)}"
		body  = {}
		files = []

	browser.goTo({ url, method, body, files })
	el.reset() unless el.hasAttribute("data-noreset")
	event.preventDefault()

module.exports = browser
